/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @package    RobotAPI::gui-plugins::DebugDrawerViewerWidgetController
 * @author     Mirko Waechter ( mirko dot waechter at kit dot edu )
 * @date       2016
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#ifndef _ARMARX_RobotAPI_DebugDrawerViewer_WidgetController_H
#define _ARMARX_RobotAPI_DebugDrawerViewer_WidgetController_H

#include "ui_DebugDrawerViewerWidget.h"

#include <ArmarXGui/libraries/ArmarXGuiBase/ArmarXGuiPlugin.h>
#include <ArmarXGui/libraries/ArmarXGuiBase/ArmarXComponentWidgetController.h>

#include <RobotAPI/components/DebugDrawer/DebugDrawerComponent.h>

#include <ArmarXCore/core/system/ImportExportComponent.h>

namespace armarx
{
    /**
    \page ArmarXGui-GuiPlugins-DebugDrawerViewer DebugDrawerViewer
    \brief The DebugDrawerViewer listens on the DebugDrawer topic and visualizes all incoming data.
    This GUI only displays the DebugDrawer content and nothing else.
    The content is shown in the 3D Viewer Widget.

    API Documentation: \ref DebugDrawerViewerWidgetController

    \see DebugDrawerViewerGuiPlugin
    */

    /**
     * \class DebugDrawerViewerWidgetController
     * \brief DebugDrawerViewerWidgetController
     *
     * Detailed description
     */
    class ARMARXCOMPONENT_IMPORT_EXPORT
        DebugDrawerViewerWidgetController:
    public armarx::ArmarXComponentWidgetController
    {
        Q_OBJECT

    public:
        /**
         * Controller Constructor
         */
        explicit DebugDrawerViewerWidgetController();

        /**
         * Controller destructor
         */
        virtual ~DebugDrawerViewerWidgetController();

        /**
         * @see ArmarXWidgetController::loadSettings()
         */
        virtual void loadSettings(QSettings* settings);

        /**
         * @see ArmarXWidgetController::saveSettings()
         */
        virtual void saveSettings(QSettings* settings);

        /**
         * Returns the Widget name displayed in the ArmarXGui to create an
         * instance of this class.
         */
        virtual QString getWidgetName() const
        {
            return "Visualization.DebugDrawerViewer";
        }

        /**
         * \see armarx::Component::onInitComponent()
         */
        virtual void onInitComponent();

        /**
         * \see armarx::Component::onConnectComponent()
         */
        virtual void onConnectComponent();

        void onExitComponent();

    public slots:
        /* QT slot declarations */

    signals:
        /* QT signal declarations */

    private:
        /**
         * Widget Form
         */
        Ui::DebugDrawerViewerWidget widget;
        armarx::DebugDrawerComponentPtr debugDrawer;
        SoSeparator* rootVisu;

        // ArmarXWidgetController interface
    public:
        SoNode* getScene();



    private slots:
        void on_btnClearAll_clicked();

        // ArmarXWidgetController interface
    public:
        QIcon getWidgetIcon() const;
    };
}

#endif
