/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @package    RobotAPI::gui-plugins::ViewSelectionWidgetController
 * @author     Markus Grotz ( markus dot grotz at kit dot edu )
 * @date       2016
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#ifndef _ARMARX_RobotAPI_ViewSelection_WidgetController_H
#define _ARMARX_RobotAPI_ViewSelection_WidgetController_H

#include "ui_ViewSelectionWidget.h"

#include <ArmarXGui/libraries/ArmarXGuiBase/ArmarXGuiPlugin.h>
#include <ArmarXGui/libraries/ArmarXGuiBase/ArmarXComponentWidgetController.h>

#include <ArmarXCore/core/system/ImportExportComponent.h>

#include <RobotAPI/interface/components/ViewSelectionInterface.h>

#include "ViewSelectionConfigDialog.h"

namespace armarx
{

    //class ViewSelectionConfigDialog;


    /**
    \page RobotAPI-GuiPlugins-ViewSelection ViewSelection
    \brief The ViewSelection allows visualizing ...

    \image html ViewSelection.png
    The user can

    API Documentation \ref ViewSelectionWidgetController

    \see ViewSelectionGuiPlugin
    */

    /**
     * \class ViewSelectionWidgetController
     * \brief ViewSelectionWidgetController brief one line description
     *
     * Detailed description
     */
    class ARMARXCOMPONENT_IMPORT_EXPORT
        ViewSelectionWidgetController:
    public armarx::ArmarXComponentWidgetController,
    public armarx::ViewSelectionObserver
    {
        Q_OBJECT

    public:
        /**
         * Controller Constructor
         */
        explicit ViewSelectionWidgetController();

        /**
         * Controller destructor
         */
        virtual ~ViewSelectionWidgetController();

        /**
         * @see ArmarXWidgetController::loadSettings()
         */
        virtual void loadSettings(QSettings* settings);

        /**
         * @see ArmarXWidgetController::saveSettings()
         */
        virtual void saveSettings(QSettings* settings);

        void configured();

        void onActivateAutomaticViewSelection(const Ice::Current& c = Ice::Current());
        void onDeactivateAutomaticViewSelection(const Ice::Current& c = Ice::Current());
        void nextViewTarget(Ice::Long, const Ice::Current& c = Ice::Current());

        /**
         * Returns the Widget name displayed in the ArmarXGui to create an
         * instance of this class.
         */
        virtual QString getWidgetName() const
        {
            return "RobotControl.ViewSelection";
        }

        /**
         * \see armarx::Component::onInitComponent()
         */
        virtual void onInitComponent();

        /**
         * \see armarx::Component::onConnectComponent()
         */
        virtual void onConnectComponent();


        virtual QPointer<QDialog> getConfigDialog(QWidget* parent);

    public slots:

        void triggerSaliencyMapVisualization();
        void clearSaliencyMapVisualization();
        void updateSaliencyMapNames();
        void toggleViewSelection(bool enabled);
        /* QT slot declarations */

    signals:
        /* QT signal declarations */

    private:
        /**
         * Widget Form
         */
        Ui::ViewSelectionWidget widget;

        QPointer<ViewSelectionConfigDialog> configDialog;

        ViewSelectionInterfacePrx viewSelection;

        std::string viewSelectionName;

    };
}

#endif
